//
//  ASMediaPlayerController.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

import AVFoundation
import Alamofire

class ASMediaPlayerController {
    
    // MARK: - 单例与状态管理
    
    static let main = ASMediaPlayerController()
    
    enum PlaybackStatus: Equatable {
        case playing
        case paused
        case stopped
        case buffering(Float)
        case volumeAdjusted(Float)
        case timeUpdated(Double)
    }
    
    private var mediaEngine: ASAudioEngine?
    
    var onStatusChange: ((PlaybackStatus) -> Void)?
    var onTimeUpdate: ((Double, Double) -> Void)?
    
    private(set) var currentStatus: PlaybackStatus = .stopped
    var totalDuration: Double {
        return mediaEngine?.trackDuration ?? 0.0
    }
    
    var isActive: Bool {
        return mediaEngine?.isActive ?? false
    }
    
    var isBuffering: Bool {
        if case .buffering(let progress) = currentStatus {
            return progress < 1.0
        }
        return false
    }
    
    var currentPosition: Double {
        return mediaEngine?.currentPosition ?? 0.0
    }
    
    var playbackProgress: Double {
        let progress = currentPosition / totalDuration
        return progress.isNaN ? 0.0 : progress
    }
    
    var isReady: Bool {
        return mediaEngine?.isReady ?? false
    }
    
    var activeMediaURL: String = ""
    var localMediaPath: URL?
    var mediaIndex: IndexPath?
    
    private var shouldCancelAfterLoad = false

    // MARK: - 播放控制
    
    func navigateTo(time: Double) {
        mediaEngine?.seek(to: time)
    }
    
    func isCurrentMedia(url: String, localPath: URL? = nil, index: IndexPath? = nil) -> Bool {
        guard activeMediaURL == url else { return false }
        
        if let currentIndex = mediaIndex,
           let newIndex = index,
           currentIndex != newIndex {
            return false
        }
        
        if let currentLocalPath = localMediaPath,
           let newLocalPath = localPath,
           currentLocalPath != newLocalPath {
            return false
        }
        
        return true
    }
    
    func prepareLocalMedia(at url: URL) {
        self.mediaEngine = ASAudioEngine(mediaURL: url)
    }
    
    func startPlayback(
        of urlString: String? = nil,
        localPath: URL? = nil,
        shouldLoop: Bool = false,
        index: IndexPath? = nil
    ) {
        stopPlayback()
        
//        guard let mediaURL = urlString else { return }
        
        activeMediaURL = urlString ?? ""
        localMediaPath = localPath
        mediaIndex = index
        let mediaURL = activeMediaURL
        let playbackHandler: (URL) -> Void = { [weak self] url in
            guard let self = self else { return }
            
            print("正在播放媒体: \(self.activeMediaURL)")
            print("媒体路径: \(url)")
            
            self.mediaEngine = ASAudioEngine(mediaURL: url)
            self.mediaEngine?.loopEnabled = shouldLoop
            
            if self.mediaEngine?.volumeLevel == 0 {
                self.adjustVolume(to: 1.0)
            }
            
            self.mediaEngine?.onTimeUpdate = { [weak self] position, duration in
                self?.onTimeUpdate?(position, duration)
                self?.updateStatus(.timeUpdated(position))
            }
            
            self.beginPlayback()
            print(self.mediaEngine?.trackDuration)
            
            self.mediaEngine?.onCompletion = { [weak self] finished in
                guard let self = self else { return }
                if finished, !(self.mediaEngine?.loopEnabled ?? false) {
                    self.stopPlayback()
                }
            }
        }
        
        shouldCancelAfterLoad = false
        
        if let localURL = self.localMediaPath, FileManager.default.fileExists(atPath: localURL.path) {
            playbackHandler(localURL)
        } else {
            self.updateStatus(.buffering(0.0))
            
            ASMediaDownloader.fetchMediaResource(
                remoteURL: mediaURL,
                progressUpdate: { progress in
                    print("下载进度: \(progress)")
                },
                completion: { [weak self] url, success in
                    guard let self = self else { return }
                    
                    self.updateStatus(.buffering(1.0))
                
                    if self.shouldCancelAfterLoad || self.activeMediaURL != mediaURL {
                        self.shouldCancelAfterLoad = false
                        return
                    }
                    
                    if let url = url {
                        playbackHandler(url)
                    } else {
                        self.stopPlayback()
                    }
                }
            )
        }
    }
    
    func beginPlayback() {
        self.mediaEngine?.play()
        updateStatus(.playing)
    }
    
    func stopPlayback() {
        self.mediaEngine?.onTimeUpdate = nil
        shouldCancelAfterLoad = true
        activeMediaURL = ""
        self.mediaEngine?.stop()
        updateStatus(.stopped)
    }
    
    func pausePlayback() {
        shouldCancelAfterLoad = true
        self.mediaEngine?.pause()
        updateStatus(.paused)
    }
    
    func adjustVolume(to level: Float) {
        self.mediaEngine?.volumeLevel = level
        updateStatus(.volumeAdjusted(level))
    }
    
    func toggleMute() -> Float {
        let newVolume:Float = (mediaEngine?.volumeLevel ?? 0.0) == 0.0 ? 1.0 : 0.0
        adjustVolume(to: newVolume)
        return newVolume
    }
    
    private func updateStatus(_ status: PlaybackStatus) {
        if case .timeUpdated(let time) = status {} else {
            print("播放状态更新: \(status)")
        }
        currentStatus = status
        self.onStatusChange?(status)
    }
    
    deinit {
        print("播放控制器已释放")
    }
}

// MARK: - 音频文件信息扩展
extension ASMediaPlayerController {
    struct MediaFileDetails {
        let fileSize: UInt64?
        let duration: Double?
        let title: String?
    }

    static func analyzeMediaFile(at path: String) -> MediaFileDetails? {
        guard let url = URL(string: path) else {
            print("无效的URL路径")
            return nil
        }
        
        guard FileManager.default.fileExists(atPath: url.path) else {
            print("文件不存在")
            return nil
        }
        
        let size = getFileSize(at: url)
        let length = getAudioDuration(from: url)
        let name = extractMediaTitle(from: url)
        
        return MediaFileDetails(
            fileSize: size,
            duration: length,
            title: name
        )
    }
    
    private static func getFileSize(at url: URL) -> UInt64? {
        do {
            let attributes = try FileManager.default.attributesOfItem(atPath: url.path)
            return attributes[.size] as? UInt64
        } catch {
            print("获取文件大小失败: \(error.localizedDescription)")
            return nil
        }
    }
    
    private static func getAudioDuration(from url: URL) -> TimeInterval? {
        var audioFile: AudioFileID?
        let status = AudioFileOpenURL(url as CFURL, .readPermission, 0, &audioFile)
        
        guard status == noErr, let file = audioFile else {
            print("音频文件打开失败: \(status)")
            return nil
        }
        
        var duration: Float64 = 0
        var propertySize = UInt32(MemoryLayout.size(ofValue: duration))
        let durationStatus = AudioFileGetProperty(
            file,
            kAudioFilePropertyEstimatedDuration,
            &propertySize,
            &duration
        )
        
        AudioFileClose(file)
        
        return durationStatus == noErr ? duration : nil
    }
    
    private static func extractMediaTitle(from fileURL: URL) -> String? {
        guard FileManager.default.fileExists(atPath: fileURL.path) else {
            print("文件不存在")
            return nil
        }
        
        let asset = AVAsset(url: fileURL)
        var title: String? = nil
        
        for format in asset.availableMetadataFormats {
            for item in asset.metadata(forFormat: format) {
                if item.commonKey == .commonKeyTitle,
                   let value = item.value as? String {
                    title = value
                    break
                }
                
                if item.key as? String == "TIT2",
                   let value = item.value as? String {
                    title = value
                    break
                }
            }
            
            if title != nil { break }
        }
        
        return title ?? fileURL.deletingPathExtension().lastPathComponent
    }
}

// MARK: - 音频引擎实现
class ASAudioEngine: NSObject {
    private var playbackEngine: AVPlayer?
    private var timeObserver: Any?
    var loopEnabled: Bool = false
    
    var isActive: Bool {
        if let engine = playbackEngine {
            if #available(iOS 10.0, *) {
                return engine.timeControlStatus == .playing
            } else {
                return engine.rate > 0
            }
        }
        return false
    }
    
    var trackDuration: Double {
        return playbackEngine?.currentItem?.asset.duration.seconds ?? 0.0
    }
    
    var volumeLevel: Float {
        get { return playbackEngine?.volume ?? 0.0 }
        set { playbackEngine?.volume = newValue.clamped(to: 0.0...1.0) }
    }
    
    var currentPosition: Double {
        return playbackEngine?.currentItem?.currentTime().seconds ?? 0.0
    }
    
    var isReady: Bool {
        return playbackEngine != nil
    }
    
    var onTimeUpdate: ((Double, Double) -> Void)?
    var onCompletion: ((Bool) -> Void)?
    
    init?(mediaURL: URL) {
        super.init()
        
        let item = AVPlayerItem(url: mediaURL)
        playbackEngine = AVPlayer(playerItem: item)
        
        NotificationCenter.default.addObserver(
            self,
            selector: #selector(playbackDidFinish),
            name: .AVPlayerItemDidPlayToEndTime,
            object: item
        )
        
        do {
            try AVAudioSession.sharedInstance().setCategory(.playback)
            try AVAudioSession.sharedInstance().setActive(true)
        } catch {
            print("音频会话配置失败: \(error.localizedDescription)")
            return nil
        }
    }
    
    deinit {
        stop()
    }
    
    func play() {
        guard let engine = playbackEngine else { return }
        
        let systemVolume = AVAudioSession.sharedInstance().outputVolume
        print("系统音量: \(systemVolume)")
        
        if systemVolume == 0.0 {
            print("请调高音量")
        }
        
        engine.play()
        observePlaybackTime()
    }
    
    func pause() {
        playbackEngine?.pause()
    }
    
    func stop() {
        playbackEngine?.pause()
        playbackEngine?.replaceCurrentItem(with: nil)
        playbackEngine = nil
        removeTimeObserver()
    }
    
    func seek(to time: Double) {
        playbackEngine?.seek(to: CMTime(seconds: time, preferredTimescale: 600))
    }
    
    @objc private func playbackDidFinish() {
        if loopEnabled {
            seek(to: 0.0)
            play()
        }
        onCompletion?(true)
    }
    
    private func observePlaybackTime() {
        let interval = CMTime(seconds: 0.1, preferredTimescale: CMTimeScale(NSEC_PER_SEC))
        timeObserver = playbackEngine?.addPeriodicTimeObserver(
            forInterval: interval,
            queue: .main
        ) { [weak self] time in
            guard let self = self else { return }
            let position = time.seconds
            self.onTimeUpdate?(position, self.trackDuration)
        }
    }
    
    private func removeTimeObserver() {
        if let observer = timeObserver {
            playbackEngine?.removeTimeObserver(observer)
            timeObserver = nil
        }
    }
}

// MARK: - 扩展工具
extension Float {
    func clamped(to range: ClosedRange<Float>) -> Float {
        return max(range.lowerBound, min(self, range.upperBound))
    }
}
