//
//  ASNetworkCoordinator+Bns.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

import Alamofire
import ObjectMapper
private let kServiceBasePath = "http://ai.soundmove.cn"

enum XHAPIEndpoint: String {
    case generateAudio = "/api/music/create"          // 音频生成接口
    case checkProgress = "/api/action/info"          // 进度查询接口
    case fileTransfer = "/api/upload"                // 文件传输接口
    
    var fullPath: String {
        return kServiceBasePath + self.rawValue
    }
}

struct ASDeviceInfo {
    static var identifier: String {
        let storageKey = "deviceUniqueID"
        if let storedID = UserDefaults.standard.string(forKey: storageKey), !storedID.isEmpty {
            return storedID
        }
        let newID = UUID().uuidString
        UserDefaults.standard.set(newID, forKey: storageKey)
        return newID
    }
    
    static var systemMetadata: [String: Any] {
        return [
            "device": deviceName,
            "deviceId": identifier,
            "iosVersion": UIDevice.current.systemVersion,
            "appVersion": appShortVersion()
        ]
    }
}


extension ASNetworkCoordinator {
    
    // MARK: - 基础请求方法
    
    func executeGET<T: ASBaseModel>(
        endpoint: XHAPIEndpoint,
        params: [String: Any]? = nil,
        modelType: T.Type? = nil,
        completion: @escaping (Any?, Error?) -> Void
    ) -> Request {
        let targetURL = endpoint.fullPath
        return performRequest(
            method: .get,
            endpoint: targetURL,
            parameters: params
        ) { result in
            switch result {
            case .success(let responseData):
                completion(responseData, nil)
            case .failure(let requestError):
                completion(nil, requestError)
            }
        }
    }
    
    func executePOST<T: ASBaseModel>(
        endpoint: XHAPIEndpoint,
        params: [String: Any]? = nil,
        modelType: T.Type? = nil,
        completion: @escaping (Any?, Error?) -> Void
    ) -> Request {
        let targetURL = endpoint.fullPath
        return performRequest(
            method: .post,
            endpoint: targetURL,
            parameters: params
        ) { result in
            switch result {
            case .success(let responseData):
                completion(responseData, nil)
            case .failure(let requestError):
                completion(nil, requestError)
            }
        }
    }
    
//    // MARK: - 文件传输方法
//    func fetchRemoteFile(
//        from endpoint: XHAPIEndpoint,
//        saveTo location: URL,
//        progressUpdate: ((Double) -> Void)? = nil,
//        completion: @escaping (URL?, Error?) -> Void
//    ) -> DownloadRequest? {
//        let targetURL = endpoint.fullPath
//        return retrieveFile(
//            from: targetURL,
//            saveTo: location,
//            progressTracker: progressUpdate
//        ) { result in
//            switch result {
//            case .success(let fileURL):
//                completion(fileURL, nil)
//            case .failure(let error):
//                completion(nil, error)
//            }
//        }
//    }
    
    // MARK: - 音频上传专用方法
    
    func sendAudioFile(
        audioFileURL: URL,
        uploadProgress: @escaping (Float) -> Void,
        completion: @escaping (Any?, Error?) -> Void
    ) -> Request? {
        let targetURL = XHAPIEndpoint.fileTransfer.fullPath
        
        debugPrint("准备上传音频文件: \(targetURL)")
        guard let validURL = URL(string: targetURL) else {
            completion(nil, NSError(domain: "NetworkError", code: 1001))
            return nil
        }
        
        return AF.upload(
            multipartFormData: { formData in
                formData.append(
                    audioFileURL,
                    withName: "file",
                    fileName: "audio.mp3",
                    mimeType: "audio/mp3"
                )
            },
            to: validURL
        )
        .uploadProgress { progress in
            debugPrint("上传进度: \(progress.fractionCompleted * 100)%")
            uploadProgress(Float(progress.fractionCompleted))
        }
        .responseString { [weak self] response in
            self?.processResponse(response) { result in
                switch result {
                case .success(let data):
                    if let json = data as? [String: Any],
                       let resultData = json["result"] as? String {
                        completion(resultData, nil)
                    } else {
                        completion(nil, NSError(domain: "DataError", code: 1002))
                    }
                case .failure(let error):
                    completion(nil, error)
                }
            }
        }
    }
}

func kNetWorkCodeSuccess(data:Any?) -> [String:Any]? {
    guard let data = data else { return nil }
    if let dataDict = data as? [String:Any],
        let code = dataDict["code"] as? Int{
        switch code {
        case 200://成功®
            return dataDict
//        case 400://机器人提示
//            TSAbnormalPopUpAlertVC.showRobotWarning()
//            return nil
        default:
            return nil
        }
    }
    return nil
}



func kNetWorkResultSuccess(data:Any?) -> [String:Any]? {
    guard let dict = kNetWorkCodeSuccess(data: data) else { return nil }
    if let result = dict["result"] as? [String:Any]{
        return result
    }
    return nil
}

func kNetWorkMessage(data:Any?) -> String? {
    guard let data = data else { return nil }
    if let dict = data as? [String:Any] ,
       let msg = dict["message"] as? String{
        return msg
    }
    return nil
}

  

class JsonStringTransform<T:Mappable>: TransformType {
    typealias Object = T
    typealias JSON = [String: Any]
    
    func transformFromJSON(_ value: Any?) -> T? {
    
        if let jsonString = value as? String {
            let obj = T(JSONString: jsonString)
            return obj
        }
        
        if let dict = value as? [String: Any] {
            let obj = T(JSON: dict)
            return obj
        }

        return nil
    }
    
    func transformToJSON(_ value: T?) -> [String : Any]? {
        return value?.toJSON() as? [String: Any]
    }
}
