//
//  ASNetworkCoordinator+error.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

import Alamofire

let kDefaultErrorMessage = "Sorry there was a slight problem with the image processing, please try again later.".localized

enum ASErrorCode: Int {
    case operationSuccess = 200
    case generalFailure = 0
    case contentRestriction = -10003
    case mediaRestriction = -10004
    case connectivityIssue = -1005
    
    var localizedDescription: String {
        switch self {
        case .contentRestriction, .mediaRestriction:
            return "Your content may contain restricted material. Please modify and retry.".localized
        case .connectivityIssue:
            return "Network unavailable. Check connection and retry.".localized
        default:
            return "Processing error occurred. Please retry later.".localized
        }
    }
    
    static func message(for statusCode: Int) -> String {
        return ASErrorCode(rawValue: statusCode)?.localizedDescription ?? ASErrorCode.generalFailure.localizedDescription
    }
    //敏感错误
    static func isContentFlagged(_ statusCode: Int) -> Bool {
        guard let code = ASErrorCode(rawValue: statusCode) else { return false }
        return code == .contentRestriction || code == .mediaRestriction
    }
    //网络错误
    static func isNetworkRelated(_ statusCode: Int) -> Bool {
        return ASErrorCode(rawValue: statusCode) == .connectivityIssue
    }
    
    static func statusCode(from error: Error) -> Int {
        if let afError = error as? AFError, let urlError = afError.underlyingError as? URLError {
            return urlError.code.networkStatusCode
        }
        if let urlError = error as? URLError {
            return urlError.code.networkStatusCode
        }
        return ASErrorCode.generalFailure.rawValue
    }
    
    static func interfaceStyle(for statusCode: Int) -> ASGeneratorView.Style {
        switch ASErrorCode(rawValue: statusCode) {
        case .contentRestriction, .mediaRestriction:
            return ASGeneratorView.Style.sensitiveError
        case .connectivityIssue:
            return ASGeneratorView.Style.netWorkError
        default:
            return ASGeneratorView.Style.generalError
        }
    }
}

extension URLError.Code {
    var networkStatusCode: Int {
        switch self {
        case .notConnectedToInternet, .networkConnectionLost, .timedOut:
            return ASErrorCode.connectivityIssue.rawValue
        default:
            return self.rawValue
        }
    }
}

extension Error {
    var xhStatusCode: Int {
        return ASErrorCode.statusCode(from: self)
    }
    
    var xhLocalizedMessage: String {
        return ASErrorCode.message(for: self.xhStatusCode)
    }
}
