//
//  ASNetworkCoordinator.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

import Alamofire
import ObjectMapper

let ASNetworkShared = ASNetworkCoordinator.shared

/// 网络请求协调器
class ASNetworkCoordinator {
    
    static let shared = ASNetworkCoordinator()
    private init() {}
    
    // MARK: - 配置属性
    
    private var standardHeaders: HTTPHeaders {
        return [
            "Content-Type": "application/json",
            "accept": "application/json"
        ]
    }
    
    lazy var sessionManager: Session = AF
    
    lazy var jsonEncoder: JSONEncoding = {
        return JSONEncoding(options: .withoutEscapingSlashes)
    }()
    
    // MARK: - 核心请求方法
    func performRequest(
        method: HTTPMethod,
        endpoint: String,
        parameters: [String: Any]? = nil,
        completion: @escaping (Result<Any, Error>) -> Void
    ) -> Request {
        debugPrint("发起网络请求: \(endpoint)")
        debugPrint("请求参数: \(parameters ?? [:])")
        
        let encoding: ParameterEncoding = method == .post ? jsonEncoder : URLEncoding.default
        
        let request = sessionManager.request(
            endpoint,
            method: method,
            parameters: parameters,
            encoding: encoding,
            headers: standardHeaders
        )
        
        request.responseString { response in
            self.processResponse(response, completion: completion)
        }
        
        return request
    }
    
    // MARK: - 文件传输方法
    func transferFile(
        endpoint: String,
        fileLocation: URL,
        formDataBuilder: @escaping (MultipartFormData) -> Void,
        progressTracker: @escaping (Progress) -> Void,
        completion: @escaping (Result<Any, Error>) -> Void
    ) -> Request? {
        debugPrint("文件传输请求: \(endpoint)")
        
        guard let targetURL = URL(string: endpoint) else {
            completion(.failure(NSError(domain: "Invalid URL", code: 1003)))
            return nil
        }
        
        return AF.upload(
            multipartFormData: formDataBuilder,
            to: targetURL
        )
        .uploadProgress { progress in
            debugPrint("传输进度: \(progress.fractionCompleted * 100)%")
            progressTracker(progress)
        }
        .responseString { response in
            self.processResponse(response, completion: completion)
        }
    }
    
    func retrieveFile(
        from endpoint: String,
        saveTo destination: URL? = nil,
        progressTracker: ((Double) -> Void)? = nil,
        completion: @escaping (URL?, Error?) -> Void
    ) -> DownloadRequest? {
        
        guard let sourceURL = URL(string: endpoint) else {
            completion(nil,NSError(domain: "Invalid URL", code: 1004))
            return nil
        }
        
        let destination: DownloadRequest.Destination = { tempURL, response in
            if let dest = destination {
                let dir = dest.deletingLastPathComponent()
                try? FileManager.default.createDirectory(at: dir, withIntermediateDirectories: true)
                return (dest, [.removePreviousFile, .createIntermediateDirectories])
            }
            
            let docsURL = FileManager.default.temporaryDirectory
            let filename = response.suggestedFilename ?? sourceURL.lastPathComponent
            return (docsURL.appendingPathComponent(filename), [.removePreviousFile, .createIntermediateDirectories])
        }
        
        let request = sessionManager.download(sourceURL, to: destination)
            .downloadProgress { progress in
                DispatchQueue.main.async {
                    progressTracker?(progress.fractionCompleted)
                }
            }
            .response { response in
                DispatchQueue.main.async {
                    switch response.result {
                    case .success(let url):
                        if let url = url {
                            completion(url,nil)
                        } else {
                            completion(nil,NSError(domain: "DownloadError", code: -1, userInfo: [NSLocalizedDescriptionKey: "无效文件路径"]))
                        }
                    case .failure(let error):
                        completion(nil,error)
                    }
                }
            }
        
        return request
    }
    
    // MARK: - 响应处理
    
    func processResponse(
        _ response: AFDataResponse<String>,
        completion: @escaping (Result<Any, Error>) -> Void
    ) {
        switch response.result {
        case .success(let value):
            if let json = convertToJSON(data: response.data) as? [String: Any] {
                handleSuccessfulResponse(data: json, response: response, completion: completion)
            } else {
                handleFailedResponse(error: NSError(domain: "DataError", code: 1005), response: response, completion: completion)
            }
        case .failure(let error):
            handleFailedResponse(error: error, response: response, completion: completion)
        }
    }
    
    private func handleSuccessfulResponse(
        data: Any,
        response: AFDataResponse<String>,
        completion: @escaping (Result<Any, Error>) -> Void
    ) {
        debugPrint("请求成功: \(response.request?.url?.absoluteString ?? "")")
        debugPrint("响应详情: \(response)")
        completion(.success(data))
    }
    
    private func handleFailedResponse(
        error: Error,
        response: AFDataResponse<String>,
        completion: @escaping (Result<Any, Error>) -> Void
    ) {
        debugPrint("请求失败: \(response.request?.url?.absoluteString ?? "")")
        debugPrint("错误详情: \(response)")
        completion(.failure(error))
    }
    
    private func convertToJSON(data: Data?) -> Any? {
        guard let data = data else { return nil }
        do {
            return try JSONSerialization.jsonObject(with: data, options: [])
        } catch {
            debugPrint("JSON转换失败: \(error.localizedDescription)")
            return nil
        }
    }
}


struct ASResponseParser {
    
    // MARK: - 响应验证
    
    /// 验证响应数据是否有效
    /// - Parameter responseData: 原始响应数据
    /// - Returns: 验证通过的字典数据
    static func validateResponsePayload(_ responseData: Any?) -> [String: Any]? {
        guard let payload = responseData,
              let jsonDict = payload as? [String: Any],
              let statusCode = jsonDict["code"] as? Int else {
            return nil
        }
        
        return statusCode == 200 ? jsonDict : nil
    }
    
    // MARK: - 数据提取
    
    /// 提取响应中的结果数据
    /// - Parameter responseData: 原始响应数据
    /// - Returns: 结果字典
    static func extractResultPayload(from responseData: Any?) -> [String: Any]? {
        guard let validResponse = validateResponsePayload(responseData),
              let resultData = validResponse["result"] as? [String: Any] else {
            return nil
        }
        return resultData
    }
    
    /// 提取响应中的消息文本
    /// - Parameter responseData: 原始响应数据
    /// - Returns: 消息字符串
    static func extractMessageText(from responseData: Any?) -> String? {
        guard let payload = responseData,
              let jsonDict = payload as? [String: Any] else {
            return nil
        }
        return jsonDict["notification"] as? String
    }
}
