//
//  ASResourceCacheHandler.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

class ASResourceCacheHandler {
    
    // MARK: - 存储路径
    
    static var ringStoragePath: URL = {
        return FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first!
            .appendingPathComponent("ring")
    }()
    
    static var universalCachePath: URL = {
        return FileManager.default.urls(for: .cachesDirectory, in: .userDomainMask).first!
            .appendingPathComponent("cacheAll")
    }()
    
    // MARK: - 文件名生成
    
    static func createHashedFilename(
        from urlString: String,
        forcedExtension: String? = nil,
        fallbackExtension: String? = nil
    ) -> String? {
        guard let sourceURL = URL(string: urlString) else { return nil }
        
        let baseName = sourceURL.path.md5
        var extensionToUse = ""
        
        if let explicitExtension = forcedExtension {
            extensionToUse = explicitExtension
        } else {
            let backupExtension = fallbackExtension ?? ""
            extensionToUse = sourceURL.pathExtension.isEmpty ? backupExtension : sourceURL.pathExtension
        }
        
        return extensionToUse.isEmpty ? baseName : "\(baseName).\(extensionToUse)"
    }
    
    // MARK: - 本地缓存查询
    
    static func locateCachedResource(
        originalURL: String,
        preferredExtension: String? = nil,
        alternativeExtension: String? = nil
    ) -> URL? {
        guard let filename = createHashedFilename(
            from: originalURL,
            forcedExtension: preferredExtension,
            fallbackExtension: alternativeExtension
        ) else { return nil }
        
        let possiblePaths = [
            ringStoragePath.appendingPathComponent(filename),
            universalCachePath.appendingPathComponent(filename)
        ]
        
        for path in possiblePaths {
            if FileManager.default.fileExists(atPath: path.path) {
                print("发现缓存资源: \(path)")
                return path
            }
        }
        
        return nil
    }
    
    // MARK: - 缓存路径处理
    
    static func prepareCacheLocation(
        for resourceURL: String,
        withExtension: String? = nil,
        defaultExtension: String? = nil,
        in directory: URL,
        completion: ((String?, Error?) -> Void)? = nil
    ) -> URL? {
        // 验证URL有效性
        guard let validatedURL = URL(string: resourceURL) else {
            completion?(nil, NSError(domain: "无效URL", code: 1001))
            return nil
        }
        
        // 处理本地路径情况
        if !resourceURL.contains("http") && resourceURL.contains("/") {
            let fullPath = resourceURL.fillCachePath
            completion?(fullPath, nil)
            return nil
        }
        
        // 确保目录存在
        let fileManager = FileManager.default
        if !fileManager.fileExists(atPath: directory.path) {
            do {
                try fileManager.createDirectory(
                    at: directory,
                    withIntermediateDirectories: true,
                    attributes: nil
                )
            } catch {
                completion?(nil, error)
                return nil
            }
        }
        
        // 生成最终文件名
        guard let finalFilename = createHashedFilename(
            from: resourceURL,
            forcedExtension: withExtension,
            fallbackExtension: defaultExtension
        ) else {
            completion?(nil, NSError(domain: "文件名生成失败", code: 1002))
            return nil
        }
        
        return directory.appendingPathComponent(finalFilename)
    }
}

// MARK: - 路径扩展
extension ASResourceCacheHandler {
    
    static func generateCachePath(
        for urlString: String,
        withExtension: String? = nil,
        defaultExtension: String? = nil,
        in directory: URL,
        completion: ((String?, Error?) -> Void)? = nil
    ) -> URL? {
        return prepareCacheLocation(
            for: urlString,
            withExtension: withExtension,
            defaultExtension: defaultExtension,
            in: directory,
            completion: completion
        )
    }
}
