//
//  UIView+AS.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

public extension UIView {

    var borderColor: UIColor? {
        set {
            layer.borderColor = newValue?.cgColor
        }
        get {
            if let cgColor = layer.borderColor {
                return UIColor(cgColor: cgColor)
            }
            return nil
        }
    }
    
    
    var x: CGFloat {
        set {
            frame.origin.x = newValue
        }
        get {
            return frame.origin.x
        }
    }
    
    var y: CGFloat {
        set {
            frame.origin.y = newValue
        }
        get {
            return frame.origin.y
        }
    }
    
    var width: CGFloat {
        set {
            frame.size.width = newValue
        }
        get {
            return frame.size.width
        }
    }
    
    var height: CGFloat {
        set {
            frame.size.height = newValue
        }
        get {
            return frame.size.height
        }
    }
    
    var size: CGSize {
        get { frame.size }
        set {
            var rect = frame
            rect.size = newValue
            frame = rect
        }
    }
    
    var centerX: CGFloat {
        set {
            let newCenter = CGPoint(x: newValue, y: center.y)
            center = newCenter
        }
        get {
            return center.x
        }
    }
    
    var centerY: CGFloat {
        set {
            let newCenter = CGPoint(x: center.x, y: newValue)
            center = newCenter
        }
        get {
            return center.y
        }
    }
    
    var bottom: CGFloat {
        get {
            return frame.origin.y + frame.size.height
        }
    }
    
    var cornerRadius: CGFloat {
        set {
            clipsToBounds = true
            layer.cornerRadius = newValue
        }
        get {
            layer.cornerRadius
        }
    }

    var borderWidth: CGFloat {
        set {
            layer.borderWidth = newValue
        }
        get {
            layer.borderWidth
        }
    }
    
}

public extension UIView {
    func removeAllSubViews() {
        subviews.forEach { subview in
            subview.removeFromSuperview()
        }
    }

    // 部分圆角（必须Frame已知）
    func maskCorner(_ corners: UIRectCorner, radius: CGFloat) {
        let path = UIBezierPath(roundedRect: bounds, byRoundingCorners: corners, cornerRadii: CGSize(width: radius, height: radius))
        let maskLayer = CAShapeLayer()
        maskLayer.frame = bounds
        maskLayer.path = path.cgPath
        layer.mask = maskLayer
    }
    
    func addGradientBorder(
        colors: [UIColor],
        width: CGFloat = 2,
        radius: CGFloat? = nil,
        startPoint: CGPoint = CGPoint(x: 0, y: 0.5),
        endPoint: CGPoint = CGPoint(x: 1, y: 0.5)
    ) {
        let gradientLayer = CAGradientLayer()
        gradientLayer.name = "GradientBorder"
        gradientLayer.endPoint = endPoint
        gradientLayer.colors = colors.map { $0.cgColor }
        gradientLayer.startPoint = startPoint
        gradientLayer.frame = bounds
        
        let cornerRadius = radius ?? layer.cornerRadius
        let path = UIBezierPath(
            roundedRect: bounds.insetBy(dx: width/2, dy: width/2),
            cornerRadius: cornerRadius
        )
        
        let shapeLayer = CAShapeLayer()
        shapeLayer.name = "GradientBorderMask"
        shapeLayer.path = path.cgPath
        shapeLayer.lineWidth = width
        shapeLayer.fillColor = nil
        shapeLayer.strokeColor = UIColor.black.cgColor
        
        gradientLayer.mask = shapeLayer
        layer.addSublayer(gradientLayer)
    }
    
    func addGradientBg(colors: [CGColor],
                       startPoint: CGPoint = .zero,
                       endPoint: CGPoint = CGPoint(x: 1, y: 1),
                       layerSize: CGSize? = nil) {
        let gl = CAGradientLayer()
        gl.colors = colors
        gl.zPosition = -1
        gl.startPoint = startPoint
        gl.endPoint = endPoint
        gl.frame = CGRect(origin: .zero, size: layerSize ?? self.bounds.size)
        layer.insertSublayer(gl, at: 0)
    }
    
    func startRotating(duration: CFTimeInterval = 1.0,
                     clockwise: Bool = true,
                     repeatCount: Float = .greatestFiniteMagnitude) {

        self.stopRotating()
        
        DispatchQueue.main.async {
            let rotation = CABasicAnimation(keyPath: "transform.rotation.z")
            rotation.repeatCount = repeatCount
            rotation.toValue = clockwise ? CGFloat.pi * 2.0 : -CGFloat.pi * 2.0
            rotation.isCumulative = true
            rotation.timingFunction = CAMediaTimingFunction(name: .linear)
            rotation.fromValue = 0
            rotation.duration = duration
            rotation.fillMode = .forwards
            rotation.isRemovedOnCompletion = false
            self.layer.add(rotation, forKey: "rotation")
        }
    }
    
    func stopRotating() {
        DispatchQueue.main.async {
            self.layer.removeAnimation(forKey: "rotation")
        }
    }
}
