//
//  ASInputView.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

class ASInputView: UITextView {
    
    // MARK: - UI Components
    private lazy var hintLabel: ASTLLabel = {
        let label = ASTLLabel()
        label.font = self.font
        label.textColor = hintTextColor
        label.text = hintText
        label.numberOfLines = 0
        label.isUserInteractionEnabled = false
        label.textAlignment = .left
        return label
    }()
    
    // MARK: - Properties
    var maximumCharacters: Int = 1000 {
        didSet {
            validateCharacterCount(inputView: self)
        }
    }
    
    var hintText: String? {
        didSet {
            hintLabel.text = hintText
        }
    }
    
    var hintTextColor: UIColor = .lightGray {
        didSet {
            hintLabel.textColor = hintTextColor
        }
    }
    
    var contentInsets: UIEdgeInsets = .zero {
        didSet {
            adjustContentInsets()
        }
    }
    
    override var text: String! {
        didSet {
            toggleHintVisibility()
        }
    }
    
    var onTextUpdate: (() -> Void)?
    
    // MARK: - Initialization
    init(hint: String? = nil,
                    initialText: String? = nil,
                    textFont: UIFont? = nil,
                    contentColor: UIColor = .black,
                    background: UIColor = .white,
                    insets: UIEdgeInsets = .zero) {
        super.init(frame: .zero, textContainer: nil)
        
        self.hintText = hint
        self.font = textFont
        self.textColor = contentColor
        self.backgroundColor = background
        self.contentInsets = insets
        
        configureHintDisplay()
        self.text = initialText
    }
    
    required init?(coder: NSCoder) {
        super.init(coder: coder)
        configureHintDisplay()
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }
    
    // MARK: - Configuration
    private func configureHintDisplay() {
        addSubview(hintLabel)
        toggleHintVisibility()
        
        NotificationCenter.default.addObserver(
            self,
            selector: #selector(handleTextUpdate(_:)),
            name: UITextView.textDidChangeNotification,
            object: self
        )
        
        adjustContentInsets()
    }
    
    // MARK: - Layout
    override func layoutSubviews() {
        super.layoutSubviews()
        
        let xPosition = contentInsets.left + 5
        let yPosition = contentInsets.top
        let labelWidth = bounds.width - contentInsets.left - contentInsets.right - 10
        let labelHeight = bounds.height - contentInsets.top - contentInsets.bottom
        
        hintLabel.frame = CGRect(
            x: xPosition,
            y: yPosition,
            width: labelWidth,
            height: labelHeight
        )
    }
    
    // MARK: - Helper Methods
    private func toggleHintVisibility() {
        hintLabel.isHidden = !text.isEmpty
    }
    
    @objc private func handleTextUpdate(_ notification: Notification) {
        if let textView = notification.object as? UITextView {
            validateCharacterCount(inputView: textView)
            onTextUpdate?()
        }
        toggleHintVisibility()
    }
    
    func validateCharacterCount(inputView: UITextView) {
        if inputView.text.count > maximumCharacters {
            inputView.text = String(inputView.text.prefix(maximumCharacters))
        }
    }
    
    private func adjustContentInsets() {
        textContainerInset = contentInsets
        setNeedsLayout()
    }
}
