//
//  FakeBlurView.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/22.
//

import UIKit


import MetalKit
import MetalPerformanceShaders

final class HybridBlurView: MTKView {
    
    // MARK: - 可配置属性
    
    /// 模糊半径 (0-30pt)
    var blurRadius: Float = 10 {
        didSet {
            metalBlurRadius = max(0, blurRadius - systemBlurRadius)
            rebuildPipeline()
        }
    }
    
    /// 原生模糊的基础半径 (0-8pt)
    var systemBlurRadius: Float = 8 {
        didSet {
            rebuildPipeline()
        }
    }
    
    // MARK: - 私有属性
    
    private var metalBlurRadius: Float = 2
    private let deviceDefault = MTLCreateSystemDefaultDevice()!
    private var commandQueue: MTLCommandQueue!
    private var textureCache: CVMetalTextureCache!
    private var blurPipeline: MPSImageGaussianBlur?
    private var sourceTexture: MTLTexture?
    private var destinationTexture: MTLTexture?
    
    private lazy var systemBlurView: UIVisualEffectView = {
        let view = UIVisualEffectView(effect: UIBlurEffect(style: .light))
        view.frame = bounds
        view.autoresizingMask = [.flexibleWidth, .flexibleHeight]
        return view
    }()
    
    // MARK: - 初始化
    
    override init(frame frameRect: CGRect, device: MTLDevice?) {
        super.init(frame: frameRect, device: device ?? MTLCreateSystemDefaultDevice()!)
        commonInit()
    }
    
    required init(coder: NSCoder) {
        super.init(coder: coder)
        commonInit()
    }
    
    private func commonInit() {
        // Metal基础配置
        self.device = device
        self.framebufferOnly = false
        self.isPaused = true
        self.enableSetNeedsDisplay = true
        self.clearColor = MTLClearColor(red: 0, green: 0, blue: 0, alpha: 0)
        
        // 创建命令队列
        commandQueue = deviceDefault.makeCommandQueue()
        
        // 设置纹理缓存
        CVMetalTextureCacheCreate(kCFAllocatorDefault, nil, deviceDefault, nil, &textureCache)
        
        // 添加系统模糊视图
        addSubview(systemBlurView)
        sendSubviewToBack(systemBlurView)
        
        // 初始管道
        rebuildPipeline()
    }
    
    // MARK: - 管道重建
    
    private func rebuildPipeline() {
        // 系统模糊视图效果调整
        let effect = UIBlurEffect(style: .light)
        systemBlurView.effect = effect
        
        // Metal模糊管道 (只处理超出系统模糊的部分)
        if metalBlurRadius > 0 {
            blurPipeline = MPSImageGaussianBlur(device: deviceDefault, sigma: metalBlurRadius)
        } else {
            blurPipeline = nil
        }
    }
    
    // MARK: - 纹理处理
    
    private func prepareTextures() {
        // 获取系统模糊视图的截图（修正cgImage访问方式）
        guard let snapshot = systemBlurView.snapshotView(afterScreenUpdates: false) else { return }

        UIGraphicsBeginImageContextWithOptions(snapshot.bounds.size, false, 0)
        defer { UIGraphicsEndImageContext() }

        guard let context = UIGraphicsGetCurrentContext() else { return }
        snapshot.layer.render(in: context)
        guard let image = UIGraphicsGetImageFromCurrentImageContext(),
              let cgImage = image.cgImage else { return }
        
        // 创建源纹理
        let textureLoader = MTKTextureLoader(device: deviceDefault)
        sourceTexture = try? textureLoader.newTexture(cgImage: cgImage, options: [
            .SRGB: false,
            .generateMipmaps: false
        ])
        
        // 创建目标纹理
        let descriptor = MTLTextureDescriptor.texture2DDescriptor(
            pixelFormat: .bgra8Unorm,
            width: Int(drawableSize.width),
            height: Int(drawableSize.height),
            mipmapped: false
        )
        descriptor.usage = [.shaderRead, .shaderWrite, .renderTarget]
        destinationTexture = deviceDefault.makeTexture(descriptor: descriptor)
    }
    
    // MARK: - 渲染
    
    override func draw(_ rect: CGRect) {
        super.draw(rect)
        
        guard let drawable = currentDrawable,
              let commandBuffer = commandQueue.makeCommandBuffer() else { return }
        
        // 确保先准备好纹理
        prepareTextures()
        
        // 如果没有额外的Metal模糊需求
        guard metalBlurRadius > 0,
              let blurPipeline = blurPipeline,
              let sourceTexture = sourceTexture,
              let destinationTexture = destinationTexture else {
            
            // 直接显示系统模糊效果（确保有源纹理）
            if let sourceTexture = sourceTexture {
                if let blitEncoder = commandBuffer.makeBlitCommandEncoder() {
                    blitEncoder.copy(from: sourceTexture, to: drawable.texture)
                    blitEncoder.endEncoding()
                }
            } else {
                // 如果连系统模糊纹理都没有，清空绘制
                let renderPassDescriptor = currentRenderPassDescriptor
                renderPassDescriptor?.colorAttachments[0].loadAction = .clear
                renderPassDescriptor?.colorAttachments[0].clearColor = clearColor
                
                if let renderEncoder = commandBuffer.makeRenderCommandEncoder(descriptor: renderPassDescriptor!) {
                    renderEncoder.endEncoding()
                }
            }
            
            commandBuffer.present(drawable)
            commandBuffer.commit()
            return
        }
        
        // 执行Metal模糊
        blurPipeline.encode(
            commandBuffer: commandBuffer,
            sourceTexture: sourceTexture,
            destinationTexture: destinationTexture
        )
        
        // 将结果复制到drawable
        if let blitEncoder = commandBuffer.makeBlitCommandEncoder() {
            blitEncoder.copy(from: destinationTexture, to: drawable.texture)
            blitEncoder.endEncoding()
        }
        
        commandBuffer.present(drawable)
        commandBuffer.commit()
    }
    
    // MARK: - 布局更新
    
    override func layoutSubviews() {
        super.layoutSubviews()
        systemBlurView.frame = bounds
        setNeedsDisplay()
    }
}

/// 仿毛玻璃效果视图（视觉欺骗方案）
final class FakeBlurView: UIView {
    
    // MARK: - 可配置属性
    
    /// 模糊强度 (0.0 ~ 1.0)
    @IBInspectable var intensity: CGFloat = 0.7 {
        didSet { updateAllLayers() }
    }
    
    /// 基础模糊色调 (默认白色)
    @IBInspectable var baseColor: UIColor = .white {
        didSet { baseLayer.backgroundColor = baseColor.withAlphaComponent(baseAlpha).cgColor }
    }
    
    /// 噪点颗粒密度 (0.1 ~ 2.0)
    @IBInspectable var noiseDensity: CGFloat = 1.0 {
        didSet { generateNoiseTexture() }
    }
    
    /// 是否启用动态光晕
    @IBInspectable var enableHighlight: Bool = true {
        didSet { highlightLayer.isHidden = !enableHighlight }
    }
    
    // MARK: - 私有属性
    
    private let baseLayer = CALayer()
    private let noiseLayer = CALayer()
    private let highlightLayer = CALayer()
    private let gradientMask = CAGradientLayer()
    private var baseAlpha: CGFloat { intensity * 0.8 }
    
    // MARK: - 初始化
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        commonInit()
    }
    
    required init?(coder: NSCoder) {
        super.init(coder: coder)
        commonInit()
    }
    
    private func commonInit() {
        layer.masksToBounds = true
        setupBaseLayer()
        setupNoiseLayer()
        setupHighlightLayer()
        setupGradientMask()
        
        // 监听设备旋转
        NotificationCenter.default.addObserver(
            self,
            selector: #selector(handleDeviceOrientation),
            name: UIDevice.orientationDidChangeNotification,
            object: nil
        )
    }
    
    // MARK: - 图层设置
    
    private func setupBaseLayer() {
        baseLayer.frame = bounds
        baseLayer.backgroundColor = baseColor.withAlphaComponent(baseAlpha).cgColor
        layer.addSublayer(baseLayer)
    }
    
    private func setupNoiseLayer() {
        noiseLayer.frame = bounds
        noiseLayer.contentsGravity = .center
        generateNoiseTexture()
        layer.addSublayer(noiseLayer)
    }
    
    private func setupHighlightLayer() {
        highlightLayer.frame = bounds
        highlightLayer.isHidden = !enableHighlight
        updateHighlight()
        layer.addSublayer(highlightLayer)
    }
    
    private func setupGradientMask() {
        gradientMask.frame = bounds
        layer.mask = gradientMask
        updateGradientMask()
    }
    
    // MARK: - 核心算法
    
    /// 生成可平铺的噪点纹理
    private func generateNoiseTexture() {
        let size = CGSize(width: 100, height: 100)
        let pointCount = Int(500 * noiseDensity)
        
        let renderer = UIGraphicsImageRenderer(size: size)
        let image = renderer.image { ctx in
            UIColor.clear.setFill()
            ctx.fill(CGRect(origin: .zero, size: size))
            
            for _ in 0..<pointCount {
                let alpha = CGFloat.random(in: 0.02...0.1)
                let point = CGPoint(
                    x: CGFloat.random(in: 0..<size.width),
                    y: CGFloat.random(in: 0..<size.height)
                )
                let radius = CGFloat.random(in: 0.5...2.0)
                
                UIColor.white.withAlphaComponent(alpha).setFill()
                UIBezierPath(ovalIn: CGRect(
                    x: point.x, y: point.y,
                    width: radius, height: radius
                )).fill()
            }
        }
        
        noiseLayer.contents = image.cgImage
        noiseLayer.contentsScale = UIScreen.main.scale
    }
    
    /// 更新径向渐变遮罩
    private func updateGradientMask() {
        gradientMask.type = .radial
        gradientMask.colors = [
            UIColor.white.cgColor,
            UIColor.clear.cgColor
        ]
        gradientMask.locations = [0, NSNumber(value: Double(intensity * 1.5))]
        
        let center = CGPoint(x: bounds.midX, y: bounds.midY)
        gradientMask.startPoint = center
        gradientMask.endPoint = CGPoint(
            x: center.x + bounds.width * 0.5,
            y: center.y + bounds.height * 0.5
        )
    }
    
    /// 更新高光效果
    private func updateHighlight() {
        let gradient = CAGradientLayer()
        gradient.frame = highlightLayer.bounds
        
        let highlightColor = UIColor.white.withAlphaComponent(0.3 * intensity)
        gradient.colors = [
            UIColor.clear.cgColor,
            highlightColor.cgColor,
            UIColor.clear.cgColor
        ]
        
        if bounds.width > bounds.height {
            gradient.startPoint = CGPoint(x: 0, y: 0.5)
            gradient.endPoint = CGPoint(x: 1, y: 0.5)
        } else {
            gradient.startPoint = CGPoint(x: 0.5, y: 0)
            gradient.endPoint = CGPoint(x: 0.5, y: 1)
        }
        
        highlightLayer.mask = gradient
    }
    
    /// 统一更新所有图层
    private func updateAllLayers() {
        baseLayer.backgroundColor = baseColor.withAlphaComponent(baseAlpha).cgColor
        updateGradientMask()
        updateHighlight()
    }
    
    // MARK: - 布局更新
    
    override func layoutSubviews() {
        super.layoutSubviews()
        baseLayer.frame = bounds
        noiseLayer.frame = bounds
        highlightLayer.frame = bounds
        gradientMask.frame = bounds
        updateAllLayers()
    }
    
    // MARK: - 设备方向处理
    
    @objc private func handleDeviceOrientation() {
        UIView.animate(withDuration: 0.3) {
            self.updateHighlight()
        }
    }
    
    // MARK: - 动态效果
    
    /// 根据滚动速度调整模糊效果
    /// - Parameter velocity: 滚动速度（来自UIScrollView）
    func adjustForScrollVelocity(_ velocity: CGPoint) {
        let velocityFactor = min(max(abs(velocity.x) + abs(velocity.y), 0), 2000) / 2000
        
        CATransaction.begin()
        CATransaction.setAnimationDuration(0.3)
        
        let scale = 1.0 + 0.3 * velocityFactor
        noiseLayer.transform = CATransform3DMakeScale(scale, scale, 1)
        highlightLayer.opacity = Float(1 - velocityFactor)
        
        CATransaction.commit()
    }
    
    // MARK: - 内存管理
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }
}

////示例1：基础使用（代码方式）
//class ViewController: UIViewController {
//    private let fakeBlurView = FakeBlurView()
//
//    override func viewDidLoad() {
//        super.viewDidLoad()
//        
//        // 添加到视图层级
//        view.addSubview(fakeBlurView)
//        fakeBlurView.frame = CGRect(x: 20, y: 100, width: 300, height: 100)
//        
//        // 基础配置
//        fakeBlurView.tintColor = .systemBlue
//        fakeBlurView.intensity = 0.8
//        fakeBlurView.noiseDensity = 1.2
//        fakeBlurView.layer.cornerRadius = 12
//        
//        // 添加测试内容
//        let label = UILabel(frame: fakeBlurView.bounds)
//        label.text = "仿毛玻璃效果测试"
//        label.textAlignment = .center
//        label.font = .boldSystemFont(ofSize: 18)
//        fakeBlurView.addSubview(label)
//    }
//}

//示例3：结合UIScrollView动态效果
//class ScrollViewController: UIViewController, UIScrollViewDelegate {
//    @IBOutlet weak var scrollView: UIScrollView!
//    @IBOutlet weak var headerBlurView: FakeBlurView!
//    
//    override func viewDidLoad() {
//        super.viewDidLoad()
//        scrollView.delegate = self
//        headerBlurView.intensity = 0.5
//    }
//    
//    func scrollViewDidScroll(_ scrollView: UIScrollView) {
//        // 根据滚动位置调整强度
//        let offsetY = scrollView.contentOffset.y
//        let maxOffset: CGFloat = 100
//        let newIntensity = min(max(0, offsetY / maxOffset), 1)
//        headerBlurView.intensity = newIntensity
//        
//        // 根据速度添加动态效果
//        let velocity = scrollView.panGestureRecognizer.velocity(in: scrollView)
//        headerBlurView.adjustForScrollVelocity(velocity)
//    }
//}

//示例4：状态栏模糊背景
//extension UIViewController {
//    func addStatusBarBlur() {
//        let statusBarView = FakeBlurView(frame: UIApplication.shared.statusBarFrame)
//        statusBarView.tag = 10086
//        statusBarView.intensity = 0.9
//        statusBarView.tintColor = .systemBackground
//        view.addSubview(statusBarView)
//    }
//    
//    func updateStatusBarBlur(alpha: CGFloat) {
//        view.viewWithTag(10086)?.alpha = alpha
//    }
//}
//

//// 在viewDidLoad中调用
//addStatusBarBlur()



// 在需要极致性能时调用
//fakeBlurView.layer.shouldRasterize = true
//fakeBlurView.layer.rasterizationScale = UIScreen.main.scale

////动态色调切换​​：
//UIView.animate(withDuration: 0.3) {
//    self.fakeBlurView.tintColor = isDarkMode ? .darkGray : .white
//}

//// 降低纹理分辨率（内存减少75%）
//fakeBlurView.noiseDensity = 0.5
